-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_LayersToFiles"

-- **************************************************
-- General information about this script
-- **************************************************

DS_LayersToFiles = {}

function DS_LayersToFiles:Name()
	return "Layers To Files"
end

function DS_LayersToFiles:Version()
	return "0.6"
end

function DS_LayersToFiles:Description()
	return "Save a new file for every layer."
end

function DS_LayersToFiles:Creator()
	return "dauid.com"
end

function DS_LayersToFiles:UILabel()
	return("Layers To Files")
end

function DS_LayersToFiles:LoadPrefs(prefs)
	self.batchOpt = prefs:GetBool("DS_LayersToFiles.batchOpt", false)
	self.numbuttOpt = prefs:GetBool("DS_LayersToFiles.numbottOpt", false)
	self.dirOpt = prefs:GetBool("DS_LayersToFiles.dirOpt", true)
	self.asDir = prefs:GetString("DS_LayersToFiles.asDir", "C:\\Program Files (x86)\\Smith Micro\\Anime Studio Pro 8\\Anime Studio Pro.exe")
	self.delOpt = prefs:GetBool("DS_LayersToFiles.delOpt", false)
end

function DS_LayersToFiles:SavePrefs(prefs)
	prefs:SetBool("DS_LayersToFiles.batchOpt", self.batchOpt)
	prefs:SetBool("DS_LayersToFiles.dirOpt", self.dirOpt)
	prefs:SetString("DS_LayersToFiles.asDir", self.asDir)
	prefs:SetBool("DS_LayersToFiles.numbuttOpt", self.numbuttOpt)
	prefs:SetBool("DS_LayersToFiles.delOpt", self.delOpt)
end


-- **************************************************
-- Recurring values
-- **************************************************


-- **************************************************
-- Settings dialogue
-- **************************************************
local RenderDialog = {}

RenderDialog.UPDATE = MOHO.MSG_BASE

function RenderDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Layers to files settings", RenderDialog)
	local l = d:GetLayout()

	d.moho = moho
	
	batchBox = LM.GUI.CheckBox("Save automatic batch file.")
	l:AddChild(batchBox, LM.GUI.ALIGN_LEFT)
	batchBox:SetValue(DS_LayersToFiles.batchOpt)
	l:AddChild(LM.GUI.StaticText("Batch options:"), LM.GUI.ALIGN_LEFT)
	
	dirBox = LM.GUI.CheckBox("Put each render in it's own folder")
	l:AddChild(dirBox, LM.GUI.ALIGN_LEFT)
	dirBox:SetValue(DS_LayersToFiles.dirOpt)
	
	numbuttBox = LM.GUI.CheckBox("Number layers in descending order")
	l:AddChild(numbuttBox, LM.GUI.ALIGN_LEFT)
	numbuttBox:SetValue(DS_LayersToFiles.numbuttOpt)
	
	l:AddChild(LM.GUI.StaticText("Directory of Anime Studio"), LM.GUI.ALIGN_LEFT)
	asDirBox = LM.GUI.TextControl(0, "C:\\Program Files\\Smith Micro\\Anime Studio Pro 8\\Anime Studio Pro.exe", RenderDialog.UPDATE, LM.GUI.FIELD_TEXT)
	l:AddChild(asDirBox)
	asDirBox:SetValue(DS_LayersToFiles.asDir)
	
	delBox = LM.GUI.CheckBox("Delete files after render")
	l:AddChild(delBox, LM.GUI.ALIGN_LEFT)
	delBox:SetValue(DS_LayersToFiles.delOpt)

	return d
end


function RenderDialog:OnValidate()
	local b = true
	return b
end

function RenderDialog:OnOK()
DS_LayersToFiles.batchOpt=batchBox:Value()
DS_LayersToFiles.dirOpt=dirBox:Value()
DS_LayersToFiles.asDir=asDirBox:Value()
DS_LayersToFiles.numbuttOpt=numbuttBox:Value()
DS_LayersToFiles.delOpt=delBox:Value()
end


-- **************************************************
-- The guts of this script
-- **************************************************

function DS_LayersToFiles:Run(moho)

local dlog = RenderDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end
minus=0
thePath=moho.document:Path()
originalName = moho.document:Name()
fileName = string.sub(originalName, 0, -6)
numLetters = string.len(originalName)+1
theDir = string.sub(thePath, 0, -numLetters)

local numlayers = moho.document:CountLayers() --Get number of layers

moho:FileSave()
--------------------------------------------------------------------------Batch stuff
if (self.batchOpt) then
	batchPath = theDir .. "\\" .. fileName .. "_render.bat"
	renderpath = theDir .. fileName .. "_render\\"
	f = io.open(batchPath, "w")
		if (f == nil) then
			return
		end
		
		f:write('@ECHO OFF \n')
			f:write('IF NOT EXIST "' .. renderpath .. '" ')
			f:write('mkdir "' .. renderpath .. '" \n')
			f:write('ECHO Starting... \n')
end
--------------------------------------------------------------------------
		
		for top = 0, numlayers - 1 do
		
		
			
				for i = 0, numlayers - 1 do
				local currLayer = moho.document:Layer(i)
					if (i == top) then
						currLayer:SetEditOnly(false)
					else
						currLayer:SetEditOnly(true)
					end
				end
				
			if (string.find(moho.document:Layer(top):Name(), "_null")==nil) then
				
				if (self.numbuttOpt) then -- number layers from top or bottom
					invertNumber=numlayers-top -- from bottom
				else
					invertNumber=top+1 -- from top
				end
				
				if (invertNumber<10) then --Add a 0 if below 10
				 invertNumber = "0" .. invertNumber 
				end
				newFile=string.sub(originalName, 0, -6) .. "_".. invertNumber .. "_" .. moho.document:Layer(top):Name() .. ".anme"
				newName=string.sub(originalName, 0, -6) .. "_".. invertNumber .. "_" .. moho.document:Layer(top):Name() 
				moho:FileSaveAs(newFile)
				
					if (self.batchOpt) then
						f:write('ECHO Rendering "' .. newName .. '" at %time% %date%\n')
						
						if (self.dirOpt) then
							thisrenderpath=renderpath .. newName
							f:write('IF NOT EXIST "' .. thisrenderpath .. '" ')
							f:write('mkdir "' .. thisrenderpath .. '" \n')
							f:write('"'.. self.asDir ..'" -r "' .. newFile .. '" -o "' .. thisrenderpath .. '\\' .. newName ..'.png" -f PNG' .. "\n")
						else
							f:write('"'.. self.asDir ..'" -r "' .. newFile .. '" -o "' .. renderpath .. '\\' .. newName ..'.png" -f PNG' .. "\n")
						end
						
						if (self.delOpt) then
							f:write('del "' .. newFile .. '"' .. "\n")
							f:write('ECHO Deleted ' .. newFile .. '\n')
						end
						
					end
			else
				minus = minus+1
			end
		end
		
		
	moho:FileOpen(thePath)
	
		
if (self.batchOpt) then
	f:write('ECHO.' .. "\n")
	f:write('ECHO Done rendering at %time% %date%' .. "\n")
	f:write('pause' .. "\n")
	f:close() --Close the file
	
end
	LM.GUI.Alert(LM.GUI.ALERT_INFO, "Saved " .. numlayers-minus.. " files.", nil, nil, "OK", nil, nil)
end
