-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_namebones"

-- **************************************************
-- General information about this script
-- **************************************************

DS_namebones = {}


function DS_namebones:Name()
	return "Name bones"
end

function DS_namebones:Version()
	return "0.1"
end

function DS_namebones:Description()
	return "Name bones"
end


function DS_namebones:Creator()
	return "dauid.com"
end

function DS_namebones:UILabel()
	return "Name bones"
end

function DS_namebones:LoadPrefs(prefs)
	self.lassoMode = prefs:GetBool("DS_namebones.lassoMode", false)
end

function DS_namebones:SavePrefs(prefs)
	prefs:SetBool("DS_namebones.lassoMode", self.lassoMode)
end

function DS_namebones:ResetPrefs()
	self.lassoMode = false
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_namebones.selBoneID = -1
DS_namebones.selRect = LM.Rect:new_local()
DS_namebones.previousX = 0
DS_namebones.previousY = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function DS_namebones:IsEnabled(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return false
		end
	end
	return true
end

function DS_namebones:IsRelevant(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR and moho.frame == 0) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return false
		end
	end
	return true
end

function DS_namebones:OnMouseDown(moho, mouseEvent)
	-- Try to select a bone by clicking on it
	self:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, mouseEvent.shiftKey, mouseEvent.ctrlKey)

	-- If no bone was selected, start dragging out a marquee selection
	local lassoMode = false
	self.ctrlKeySelection = false
	if (self.lassoMode) then
		if (not(mouseEvent.ctrlKey)) then
			lassoMode = true
		else
			self.ctrlKeySelection = true
		end
	else
		if (mouseEvent.ctrlKey) then
			lassoMode = true
			self.ctrlKeySelection = true
		end
	end

	mouseEvent.view:DrawMe()
	if (self.selBoneID == -1) then
		if (lassoMode) then
			self.lassoList = { { mouseEvent.startPt.x, mouseEvent.startPt.y } }
			self.previousX = mouseEvent.startPt.x
			self.previousY = mouseEvent.startPt.y
		else
			self.selRect.left = mouseEvent.startPt.x
			self.selRect.top = mouseEvent.startPt.y
			self.selRect.right = mouseEvent.pt.x
			self.selRect.bottom = mouseEvent.pt.y
			mouseEvent.view:Graphics():SelectionRect(self.selRect)
		end
	end
end

function DS_namebones:OnMouseMoved(moho, mouseEvent)
	--self:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, mouseEvent.shiftKey, mouseEvent.ctrlKey)
	if (self.selBoneID >= 0) then
		return
	end

	local lassoMode = false
	if (self.lassoMode) then
		if (not(self.ctrlKeySelection)) then
			lassoMode = true
		end
	else
		if (self.ctrlKeySelection) then
			lassoMode = true
		end
	end

	if (lassoMode) then
		local g = mouseEvent.view:Graphics()

		g:SetSmoothing(true)
		g:Push()
		local m = g:CurrentTransform()
		m:Invert()
		g:ApplyMatrix(m)
		g:SetColor(MOHO.MohoGlobals.SelCol)
		g:MoveTo(self.previousX, self.previousY)
		g:LineTo(mouseEvent.pt.x, mouseEvent.pt.y)
		g:Pop()
		g:SetSmoothing(false)
		mouseEvent.view:RefreshView()

		table.insert(self.lassoList, { mouseEvent.pt.x, mouseEvent.pt.y })
		self.previousX = mouseEvent.pt.x
		self.previousY = mouseEvent.pt.y
	else
		mouseEvent.view:Graphics():SelectionRect(self.selRect)
		self.selRect.right = mouseEvent.pt.x
		self.selRect.bottom = mouseEvent.pt.y
		mouseEvent.view:Graphics():SelectionRect(self.selRect)
		mouseEvent.view:RefreshView()
	end
end

function DS_namebones:OnMouseUp(moho, mouseEvent)
	if (self.selBoneID >= 0) then
		return
	end

	local lassoMode = false
	if (self.lassoMode) then
		if (not(self.ctrlKeySelection)) then
			lassoMode = true
		end
	else
		if (self.ctrlKeySelection) then
			lassoMode = true
		end
	end

	if (self.selMode == self.SELMODE_GROUP_PTS and lassoMode) then
		-- draw the finalized lasso outline
		local g = mouseEvent.view:Graphics()

		g:SetSmoothing(true)
		g:Push()
		local m = g:CurrentTransform()
		m:Invert()
		g:ApplyMatrix(m)
		g:SetColor(MOHO.MohoGlobals.SelCol)
		g:MoveTo(self.previousX, self.previousY)
		g:LineTo(mouseEvent.startPt.x, mouseEvent.startPt.y)
		g:Pop()
		g:SetSmoothing(false)
		mouseEvent.view:RefreshView()
		LM.Snooze(100)
	end

	-- 1 - Draw the selection shape, either a lasso or a rectangle
	local end1 = LM.Vector2:new_local()
	local end2 = LM.Vector2:new_local()
	local g = mouseEvent.view:Graphics()
	g:Clear(0, 0, 0, 0)
	g:Push()
	local m = g:CurrentTransform()
	m:Invert()
	g:ApplyMatrix(m)
	g:SetColor(255, 255, 255, 255)
	g:BeginShape()
	if (lassoMode) then
		for i = 1, table.getn(self.lassoList) - 1 do
			end1:Set(self.lassoList[i][1], self.lassoList[i][2])
			end2:Set(self.lassoList[i + 1][1], self.lassoList[i + 1][2])
			g:AddLine(end1, end2)
		end
		end1:Set(self.lassoList[table.getn(self.lassoList)][1], self.lassoList[table.getn(self.lassoList)][2])
		end2:Set(self.lassoList[1][1], self.lassoList[1][2])
		g:AddLine(end1, end2)
	else
		end1:Set(mouseEvent.startPt.x, mouseEvent.startPt.y)
		end2:Set(mouseEvent.pt.x, mouseEvent.startPt.y)
		g:AddLine(end1, end2)
		
		end1:Set(end2.x, end2.y)
		end2:Set(mouseEvent.pt.x, mouseEvent.pt.y)
		g:AddLine(end1, end2)
		
		end1:Set(end2.x, end2.y)
		end2:Set(mouseEvent.startPt.x, mouseEvent.pt.y)
		g:AddLine(end1, end2)
		
		end1:Set(end2.x, end2.y)
		end2:Set(mouseEvent.startPt.x, mouseEvent.startPt.y)
		g:AddLine(end1, end2)
	end
	g:EndShape()
	g:Pop()
	-- test code to view the lasso's shape
	--mouseEvent.view:RefreshView()
	--LM.Snooze(1000)

	-- 2 - Do hit testing on the selection shape
	local v = LM.Vector2:new_local()
	local screenPt = LM.Point:new_local()
	local m = LM.Matrix:new_local()
	local parentSkeleton = false
	local skel = moho:Skeleton()
	moho.layer:GetFullTransform(moho.frame, m, moho.document)
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
			parentSkeleton = true
			moho.layer:Parent():GetFullTransform(moho.frame, m, moho.document)
		end
	end

	if (skel ~= nil) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			local boneMatrix = bone.fMovedMatrix
			if (moho.frame == 0) then
				boneMatrix = bone.fRestMatrix
			end
			for j = 0, 10 do
				v:Set(bone.fLength * j / 10.0, 0)
				boneMatrix:Transform(v)
				m:Transform(v)
				g:WorldToScreen(v, screenPt)
				if (g:IsFullWhite(screenPt)) then
					if (mouseEvent.shiftKey) then
						bone.fSelected = not bone.fSelected
					else
						bone.fSelected = true
					end
					break
				end
			end
		end
	end

	self.lassoList = nil

	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

function DS_namebones:Select(moho, mousePt, mouseVec, mouseView, shiftSelect, ctrlSelect)
	self.selBoneID = -1

	local parentSkeleton = false
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
			parentSkeleton = true
		end
		if (skel == nil) then
			return
		end
	end

	local id = -1
	if (parentSkeleton) then
		id = mouseView:PickBone(mousePt, mouseVec, moho.layer:Parent(), true)
	else
		id = mouseView:PickBone(mousePt, mouseVec, moho.layer, true)
	end
	self.selBoneID = id

	if (shiftSelect) then
		if (id >= 0) then
			skel:Bone(id).fSelected = true
		end
	elseif (ctrlSelect) then
		if (id >= 0) then
			skel:Bone(id).fSelected = not skel:Bone(id).fSelected
		end
	else
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == id)
		end
	end

	moho:UpdateBonePointSelection()
	mouseView:DrawMe()
	moho:UpdateSelectedChannels()
end

function DS_namebones:OnKeyDown(moho, keyEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return
		end
	end

	if (keyEvent.keyCode == LM.GUI.KEY_UP and not keyEvent.ctrlKey) then
		for i = 0, skel:CountBones() - 1 do
			if (skel:Bone(i).fSelected) then
				local pID = skel:Bone(i).fParent
				if (pID >= 0) then
					skel:Bone(i).fSelected = false
					skel:Bone(pID).fSelected = true
				end
				break
			end
		end
		moho:UpdateBonePointSelection()
		keyEvent.view:DrawMe()
		moho:UpdateSelectedChannels()
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN and not keyEvent.ctrlKey) then
		for i = 0, skel:CountBones() - 1 do
			if (skel:Bone(i).fSelected) then
				local childID = -1
				if (skel:CountBoneChildren(i) == 1) then
					childID = skel:GetFirstChildBone(i)
				end
				if (childID >= 0) then
					skel:Bone(i).fSelected = false
					skel:Bone(childID).fSelected = true
				end
				break
			end
		end
		moho:UpdateBonePointSelection()
		keyEvent.view:DrawMe()
		moho:UpdateSelectedChannels()
	elseif ((keyEvent.keyCode == LM.GUI.KEY_DELETE) or (keyEvent.keyCode == LM.GUI.KEY_BACKSPACE)) then
		if (moho.layer:IsBoneType() and (moho.layerFrame == 0 or not moho:DisableDrawingTools())) then
			local boneLayer = moho:LayerAsGroup(moho.layer)
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			keepTrying = true
			while keepTrying do
				keepTrying = false
				for i = 0, skel:CountBones() - 1 do
					if (skel:Bone(i).fSelected) then
						for j = 0, boneLayer:CountLayers() - 1 do
							boneLayer:Layer(j):DeleteParentBone(i)
						end
						skel:DeleteBone(i)
						keepTrying = true
						break
					end
				end
			end
			keyEvent.view:DrawMe()
		end
	end
end

-- **************************************************
-- Bone constraints dialog
-- **************************************************

local DS_namebonesDialog = {}

function DS_namebonesDialog:new()
	local d = LM.GUI.SimpleDialog(MOHO.Localize("/Scripts/Tool/SelectBone/BoneConstraints=Bone Constraints"), DS_namebonesDialog)
	local l = d:GetLayout()

	l:PushV(LM.GUI.ALIGN_LEFT, 0)

	d.angleConstraints = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/SelectBone/AngleConstraints=Angle constraints:"), DS_namebones.DLOG_CHANGE)
	l:AddChild(d.angleConstraints, LM.GUI.ALIGN_LEFT)
	l:PushH()
		l:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/SelectBone/MinMaxDegrees=Min/max (degrees)")))
		d.minAngle = LM.GUI.TextControl(0, "0000.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		d.minAngle:SetWheelInc(5)
		l:AddChild(d.minAngle)
		d.maxAngle = LM.GUI.TextControl(0, "0000.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		d.maxAngle:SetWheelInc(5)
		l:AddChild(d.maxAngle)
	l:Pop()

	l:AddPadding()

	l:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/SelectBone/AngleControlBone=Angle control bone:")), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.acMenu = LM.GUI.Menu(MOHO.Localize("/Scripts/Tool/SelectBone/SelectBone=Select Bone"))
		d.acPopup = LM.GUI.PopupMenu(128, true)
		d.acPopup:SetMenu(d.acMenu)
		l:AddChild(d.acPopup)
		d.angleControl = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.angleControl)
	l:Pop()

	l:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/SelectBone/PositionControlBone=Position control bone:")), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.pcMenu = LM.GUI.Menu(MOHO.Localize("/Scripts/Tool/SelectBone/SelectBone=Select Bone"))
		d.pcPopup = LM.GUI.PopupMenu(128, true)
		d.pcPopup:SetMenu(d.pcMenu)
		l:AddChild(d.pcPopup)
		d.posControlX = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.posControlX)
		d.posControlY = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.posControlY)
	l:Pop()

	l:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/SelectBone/ScaleControlBone=Scale control bone:")), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.scMenu = LM.GUI.Menu(MOHO.Localize("/Scripts/Tool/SelectBone/SelectBone=Select Bone"))
		d.scPopup = LM.GUI.PopupMenu(128, true)
		d.scPopup:SetMenu(d.scMenu)
		l:AddChild(d.scPopup)
		d.scaleControl = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_FLOAT)
		l:AddChild(d.scaleControl)
	l:Pop()

	d.boneDynamics = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/SelectBone/BoneDynamics=Bone dynamics:"), DS_namebones.DLOG_CHANGE)
	l:AddChild(d.boneDynamics, LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/SelectBone/TorqueSpringDampingForce=Torque force / Spring force / Damping force")), LM.GUI.ALIGN_LEFT)
	l:PushH()
		d.torque = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		d.torque:SetWheelInc(0.1)
		l:AddChild(d.torque)
		d.spring = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		d.spring:SetWheelInc(0.1)
		l:AddChild(d.spring)
		d.damping = LM.GUI.TextControl(0, "00.0000", DS_namebones.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		d.damping:SetWheelInc(0.1)
		l:AddChild(d.damping)
	l:Pop()

	l:PushH(LM.GUI.ALIGN_RIGHT, 0)
		l:AddChild(LM.GUI.Button(MOHO.Localize("/Scripts/Tool/SelectBone/Close=Close"), LM.GUI.MSG_CANCEL))
	l:Pop()

	l:Pop()

	return d
end

function DS_namebonesDialog:UpdateWidgets()
	if (self.document and self.layer and self.skel and self.bone) then
		self.angleConstraints:Enable(true)
		self.acPopup:Enable(true)
		self.pcPopup:Enable(true)
		self.scPopup:Enable(true)
		self.boneDynamics:Enable(true)

		self.angleConstraints:SetValue(self.bone.fConstraints)
		self.minAngle:SetValue(math.deg(self.bone.fMinConstraint))
		self.maxAngle:SetValue(math.deg(self.bone.fMaxConstraint))
		self.minAngle:Enable(self.bone.fConstraints)
		self.maxAngle:Enable(self.bone.fConstraints)

		local id = self.skel:BoneID(self.bone)

		MOHO.BuildBoneChoiceMenu(self.acMenu, self.skel, DS_namebones.SELECTCONTROL, id)
		if (self.bone.fAngleControlParent < 0) then
			self.acMenu:SetChecked(DS_namebones.SELECTCONTROL, true)
		else
			self.acMenu:SetChecked(DS_namebones.SELECTCONTROL + 1 + self.bone.fAngleControlParent, true)
		end
		self.acPopup:Redraw()

		MOHO.BuildBoneChoiceMenu(self.pcMenu, self.skel, DS_namebones.SELECTCONTROL, id)
		if (self.bone.fPosControlParent < 0) then
			self.pcMenu:SetChecked(DS_namebones.SELECTCONTROL, true)
		else
			self.pcMenu:SetChecked(DS_namebones.SELECTCONTROL + 1 + self.bone.fPosControlParent, true)
		end
		self.pcPopup:Redraw()

		MOHO.BuildBoneChoiceMenu(self.scMenu, self.skel, DS_namebones.SELECTCONTROL, id)
		if (self.bone.fScaleControlParent < 0) then
			self.scMenu:SetChecked(DS_namebones.SELECTCONTROL, true)
		else
			self.scMenu:SetChecked(DS_namebones.SELECTCONTROL + 1 + self.bone.fScaleControlParent, true)
		end
		self.scPopup:Redraw()

		self.angleControl:Enable(self.bone.fAngleControlParent >= 0)
		self.posControlX:Enable(self.bone.fPosControlParent >= 0)
		self.posControlY:Enable(self.bone.fPosControlParent >= 0)
		self.scaleControl:Enable(self.bone.fScaleControlParent >= 0)

		self.angleControl:SetValue(self.bone.fAngleControlScale)
		self.posControlX:SetValue(self.bone.fPosControlScale.x)
		self.posControlY:SetValue(self.bone.fPosControlScale.y)
		self.scaleControl:SetValue(self.bone.fScaleControlScale)

		self.boneDynamics:SetValue(self.bone.fBoneDynamics.value)
		self.torque:SetValue(self.bone.fTorqueForce)
		self.spring:SetValue(self.bone.fSpringForce)
		self.damping:SetValue(self.bone.fDampingForce)
		self.torque:Enable(self.bone.fBoneDynamics.value)
		self.spring:Enable(self.bone.fBoneDynamics.value)
		self.damping:Enable(self.bone.fBoneDynamics.value)
	else
		self.angleConstraints:Enable(false)
		self.minAngle:Enable(false)
		self.maxAngle:Enable(false)

		self.acPopup:Enable(false)
		self.angleControl:Enable(false)

		self.pcPopup:Enable(false)
		self.posControlX:Enable(false)
		self.posControlY:Enable(false)

		self.scPopup:Enable(false)
		self.scaleControl:Enable(false)

		self.boneDynamics:Enable(false)
		self.torque:Enable(false)
		self.spring:Enable(false)
		self.damping:Enable(false)
	end
end

function DS_namebonesDialog:OnOK()
	self:HandleMessage(DS_namebones.DLOG_CHANGE) -- send this final message in case the user is in the middle of editing some value
end

function DS_namebonesDialog:HandleMessage(msg)
	if (not (self.document and self.layer and self.skel and self.bone)) then
		return
	end

	if ((msg == DS_namebones.DLOG_CHANGE) or (msg == LM.GUI.MSG_CANCEL)) then
		self.document:PrepUndo(self.layer)
		self.document:SetDirty()

		for i = 0, self.skel:CountBones() - 1 do
			local bone = self.skel:Bone(i)
			if (bone.fSelected) then
				bone.fConstraints = self.angleConstraints:Value()
				bone.fMinConstraint = math.rad(self.minAngle:FloatValue())
				bone.fMaxConstraint = math.rad(self.maxAngle:FloatValue())
				if (self.minAngle:IsEnabled() ~= bone.fConstraints) then
					self.minAngle:Enable(bone.fConstraints)
					self.maxAngle:Enable(bone.fConstraints)
				end

				bone.fAngleControlScale = self.angleControl:FloatValue()
				bone.fPosControlScale.x = self.posControlX:FloatValue()
				bone.fPosControlScale.y = self.posControlY:FloatValue()
				bone.fScaleControlScale = self.scaleControl:FloatValue()

				if (bone.fBoneDynamics.value ~= self.boneDynamics:Value()) then
					bone.fBoneDynamics.value = self.boneDynamics:Value()
					bone.fBoneDynamics:StoreValue()
					self.layer:UpdateCurFrame(true)
					MOHO.NewKeyframe(CHANNEL_BONE_DYNAMICS)
					MOHO.NewKeyframe(CHANNEL_BONE_SEL_DYNAMICS)
				end
				bone.fTorqueForce = self.torque:FloatValue()
				bone.fSpringForce = self.spring:FloatValue()
				bone.fDampingForce = self.damping:FloatValue()
				if (self.torque:IsEnabled() ~= bone.fBoneDynamics.value) then
					self.torque:Enable(bone.fBoneDynamics.value)
					self.spring:Enable(bone.fBoneDynamics.value)
					self.damping:Enable(bone.fBoneDynamics.value)
				end
			end
		end

		MOHO.Redraw()
	elseif (msg >= DS_namebones.SELECTCONTROL) then
		self.document:PrepUndo(self.layer)
		self.document:SetDirty()

		for i = 0, self.skel:CountBones() - 1 do
			local bone = self.skel:Bone(i)
			if (bone.fSelected) then
				local ctrlID = self.acMenu:FirstCheckedMsg() - (DS_namebones.SELECTCONTROL + 1)
				if (ctrlID ~= self.skel:BoneID(bone)) then
					bone.fAngleControlParent = ctrlID
				end
				ctrlID = self.pcMenu:FirstCheckedMsg() - (DS_namebones.SELECTCONTROL + 1)
				if (ctrlID ~= self.skel:BoneID(bone)) then
					bone.fPosControlParent = ctrlID
				end
				ctrlID = self.scMenu:FirstCheckedMsg() - (DS_namebones.SELECTCONTROL + 1)
				if (ctrlID ~= self.skel:BoneID(bone)) then
					bone.fScaleControlParent = ctrlID
				end

				self.angleControl:Enable(bone.fAngleControlParent >= 0)
				self.posControlX:Enable(bone.fPosControlParent >= 0)
				self.posControlY:Enable(bone.fPosControlParent >= 0)
				self.scaleControl:Enable(bone.fScaleControlParent >= 0)
			end
		end
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_namebones.CHANGE = MOHO.MSG_BASE
DS_namebones.DLOG_BEGIN = MOHO.MSG_BASE + 1
DS_namebones.DLOG_CHANGE = MOHO.MSG_BASE + 2
DS_namebones.LOCK = MOHO.MSG_BASE + 3
DS_namebones.LASSO = MOHO.MSG_BASE + 4
DS_namebones.DUMMY = MOHO.MSG_BASE + 5
DS_namebones.SELECTITEM = MOHO.MSG_BASE + 6
DS_namebones.FRONTHAND = MOHO.MSG_BASE+7
DS_namebones.BACKHAND = MOHO.MSG_BASE +8
DS_namebones.FRONTTOE = MOHO.MSG_BASE +9
DS_namebones.BACKTOE = MOHO.MSG_BASE +10
DS_namebones.HEAD = MOHO.MSG_BASE +11


DS_namebones.SELECTCONTROL = MOHO.MSG_BASE + 300

function DS_namebones:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu(MOHO.Localize("/Scripts/Tool/SelectBone/SelectBone=Select Bone"))

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	--layout:AddChild(self.popup)

	self.boneName = LM.GUI.TextControl(0, "Room For a Big Long Name", self.CHANGE, LM.GUI.FIELD_TEXT)
	self.boneName:SetValue("")
	layout:AddChild(self.boneName)

	self.dlog = DS_namebonesDialog:new()
	self.popup = LM.GUI.PopupDialog(MOHO.Localize("/Scripts/Tool/SelectBone/ConeConstraints=Bone Constraints"), true, self.DLOG_BEGIN)
	self.popup:SetDialog(self.dlog)
	--layout:AddChild(self.popup)

	self.lockBone = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/SelectBone/LockBone=Lock bone"), self.LOCK)
	--layout:AddChild(self.lockBone)

	self.lassoCheck = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/SelectBone/LassoMode=Lasso mode"), self.LASSO)
	--layout:AddChild(self.lassoCheck)
	
	self.fronthandbut= LM.GUI.Button("Front hand", self.FRONTHAND)
	layout:AddChild(self.fronthandbut)	
	self.backhandbut= LM.GUI.Button("Back hand", self.BACKHAND)
	layout:AddChild(self.backhandbut)
	self.fronttoebut= LM.GUI.Button("Front toe", self.FRONTTOE)
	layout:AddChild(self.fronttoebut)
	self.backtoebut= LM.GUI.Button("Back toe", self.BACKTOE)
	layout:AddChild(self.backtoebut)
	self.headbut= LM.GUI.Button("Head", self.HEAD)
	layout:AddChild(self.headbut)
end

function DS_namebones:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return
		end
	end

	local selID = skel:SelectedBoneID()

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (selID >= 0) then
		local bone = skel:Bone(selID)
		self.boneName:SetValue(bone:Name())
		self.popup:Enable(true)
		self.dlog.bone = bone
		self.lockBone:Enable(true)
		self.lockBone:SetValue(bone.fIKLock.value)
	else
		self.boneName:SetValue("")
		self.popup:Enable(false)
		self.dlog.bone = nil
		self.lockBone:Enable(false)
		self.lockBone:SetValue(false)
	end

	self.lassoCheck:SetValue(self.lassoMode)

	self.dlog.document = moho.document
	self.dlog.layer = moho.layer
	self.dlog.skel = skel
	self.dlog:UpdateWidgets()
end

function DS_namebones:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	local layer = moho.layer
	if (skel == nil) then
		if (moho.layer:LayerType() == MOHO.LT_VECTOR) then -- this tool can also be used when a vector layer is active
			skel = moho:ParentSkeleton()
		end
		if (skel == nil) then
			return
		end
		layer = moho.layer:Parent()
	end
	if (msg == self.FRONTHAND) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				bone:SetName("Front hand")
				local lowerarm = skel:Bone(bone.fParent)
				lowerarm:SetName("Front lower arm")
				local upperarm = skel:Bone(lowerarm.fParent)
				upperarm:SetName("Front upper arm")
			end
		end
	end
	
	if (msg == self.BACKHAND) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				bone:SetName("Back hand")
				local lowerarm = skel:Bone(bone.fParent)
				lowerarm:SetName("Back lower arm")
				local upperarm = skel:Bone(lowerarm.fParent)
				upperarm:SetName("Back upper arm")
			end
		end
	end
	if (msg == self.BACKTOE) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				
				bone:SetName("Back lower foot")
				local upperfoot = skel:Bone(bone.fParent)
				upperfoot:SetName("Back upper foot")
				local lowerleg = skel:Bone(upperfoot.fParent)
				lowerleg:SetName("Back lower leg")
				local upperleg = skel:Bone(lowerleg.fParent)
				upperleg:SetName("Back upper leg")
			end
		end
	end
	if (msg == self.FRONTTOE) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				bone:SetName("Front lower foot")
				local upperfoot = skel:Bone(bone.fParent)
				upperfoot:SetName("Front upper foot")
				local lowerleg = skel:Bone(upperfoot.fParent)
				lowerleg:SetName("Front lower leg")
				local upperleg = skel:Bone(lowerleg.fParent)
				upperleg:SetName("Front upper leg")
			end
		end
	end
	
	if (msg == self.HEAD) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				bone:SetName("Head")
				local neck = skel:Bone(bone.fParent)
				neck:SetName("Neck")
				local uppertorso = skel:Bone(neck.fParent)
				uppertorso:SetName("Upper torso")
				local lowertorso = skel:Bone(uppertorso.fParent)
				lowertorso:SetName("Lower torso")
				
				if skel:Bone(lowertorso.fParent).fParent == -1 then	
					local master = skel:Bone(lowertorso.fParent)
					master:SetName("Master")
				else
					local lowerlowertorso = skel:Bone(lowertorso.fParent)
					lowerlowertorso:SetName("Lower lower torso")
					local master = skel:Bone(lowerlowertorso.fParent)
					master:SetName("Master")
				end
			end
		end
	end
	
	

	if (msg == self.CHANGE) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone:SetName(self.boneName:Value())
				end
			end
		end
	elseif (msg == self.LOCK) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					if (self.lockBone:Value()) then
						skel:LockBone(i, moho.layerFrame)
					else
						skel:UnlockBone(i, moho.layerFrame)
					end
				end
			end
			moho.layer:UpdateCurFrame(true)
			moho:NewKeyframe(CHANNEL_BONE)
			moho:NewKeyframe(CHANNEL_BONE_LOCK)
		end
	elseif (msg == self.LASSO) then
		self.lassoMode = self.lassoCheck:Value()
	elseif (msg == self.DLOG_BEGIN) then
		self.dlog.document = moho.document
		self.dlog.layer = layer
		self.dlog.skel = skel
		if (skel:SelectedBoneID() >= 0) then
			self.dlog.bone = skel:Bone(skel:SelectedBoneID())
		else
			self.dlog.bone = nil
		end
	elseif (msg == self.DLOG_CHANGE) then
		-- Nothing really happens here - it is a message that came from the popup dialog.
		-- However, the important thing is that this message then flows back into the Moho app, forcing a redraw.
	elseif (msg >= self.SELECTITEM and msg < self.SELECTCONTROL) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateBonePointSelection()
		moho:UpdateUI()
	end
end
